package org.bss.esb.test;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.QueueReceiver;
import javax.jms.TextMessage;
import javax.jms.Queue;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * @author Balwinder Sodhi
 * @version 1.0
 */
public class JmsReceiver implements MessageListener {

    private QueueConnection connection;
    private QueueConnectionFactory fact;
    private QueueSession session;
    private static String selector = null;

    public JmsReceiver(String url, String user, String passwd,
            String connFact, String queueName) throws NamingException, JMSException {

        Hashtable env = new Hashtable();
        env.put(InitialContext.INITIAL_CONTEXT_FACTORY,
                "weblogic.jndi.WLInitialContextFactory");
        env.put(InitialContext.SECURITY_PRINCIPAL, user);
        env.put(InitialContext.SECURITY_CREDENTIALS, passwd);
        env.put(InitialContext.PROVIDER_URL, url);
        InitialContext ic = new InitialContext(env);
        fact = (QueueConnectionFactory) ic.lookup(connFact);
        connection = fact.createQueueConnection();
        session = connection.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
        Queue q = (Queue) ic.lookup(queueName);
        QueueReceiver recevier = session.createReceiver(q, selector);
        recevier.setMessageListener(this);
        connection.start();
        System.out.println("Started connection ...");
    }

    public void onMessage(Message message) {


        try {

            System.out.println("\n\nSelector: "+selector);
            StringBuffer sb = new StringBuffer();
            sb.append("\n================= JMS Headers =====================");
            sb.append("\nJMSRedelivered: ").append(message.getJMSRedelivered());
            sb.append("\nJMSTimestamp: ").append(message.getJMSTimestamp());
            sb.append("\nJMSPriority: ").append(message.getJMSPriority());
            sb.append("\nJMSMessageID: ").append(message.getJMSMessageID());
            sb.append("\nJMSExpiration: ").append(message.getJMSExpiration());
            sb.append("\nJMSDeliveryMode: ").append(message.getJMSDeliveryMode());
            sb.append("\nJMSCorrelationID: ").append(message.getJMSCorrelationID());
            Enumeration en = message.getPropertyNames();
            sb.append("\n================= JMS Properties ==================");
            while (en.hasMoreElements()) {
                Object o =  en.nextElement();
                sb.append("\n").append(o).append(" = ");
                sb.append(message.getObjectProperty((String)o));
            }
            sb.append("\n================= Message Body ====================");
            sb.append("\n").append(((TextMessage)message).getText());
            System.out.println(sb.toString());

        } catch (JMSException jme) {
            jme.printStackTrace();
        }
    }

    public static void main(String[] args) throws JMSException, NamingException {
        if (args.length < 5) {
            System.out.println("Usage: java "+JmsReceiver.class.getName()+
                    " JndiUrl User Password Queue QueueConnFactory [selector]");
            System.exit(-1);
        }
        String url = args[0];
        String user = args[1];
        String passwd = args[2];
        String qcf = args[4];
        String queue = args[3];
        if (args.length == 6) {
            selector = args[5];

        }
        JmsReceiver recv = new JmsReceiver(url, user, passwd, qcf, queue);
        System.out.println("Receiver started.");
        synchronized(recv) {
            while (true) {
                try {
                    recv.wait();
                } catch (InterruptedException e) {

                }
            }
        }
    }

}
