package org.bss.esb.test;

import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.jms.Queue;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Hashtable;

public class JmsSender {

    private QueueConnectionFactory qconFactory;
    private QueueConnection qcon;
    private QueueSession qsession;
    private QueueSender qsender;
    private TextMessage msg;

    /**
     * @param args
     * @throws NamingException
     * @throws JMSException
     */
    public void init(String[] args) throws NamingException, JMSException {


        Hashtable env = new Hashtable();
        env.put(InitialContext.INITIAL_CONTEXT_FACTORY,
                "weblogic.jndi.WLInitialContextFactory");
        env.put(InitialContext.SECURITY_PRINCIPAL, args[1]);
        env.put(InitialContext.SECURITY_CREDENTIALS, args[2]);
        env.put(InitialContext.PROVIDER_URL, args[0]);
        InitialContext ic = new InitialContext(env);
        qconFactory = (QueueConnectionFactory) ic.lookup(args[4]);
        qcon = qconFactory.createQueueConnection();
        qsession = qcon.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
        Queue q = (Queue) ic.lookup(args[3]);
        qsender = qsession.createSender(q);
        msg = qsession.createTextMessage();
        qcon.start();
    }

    /**
     * Sends a message to a JMS queue.
     *
     * @param message message to be sent
     * @throws JMSException if JMS fails to send message due to internal error
     */
    public void send(String message) throws JMSException {

        String[] parts = message.split("@");
        String[] props = parts[1].split(",");
        for (int i = 0; i < props.length; i++) {

            String[] p = props[i].split("=");
            msg.setStringProperty(p[0], p[1]);
            
        }

        msg.setText(parts[0]);
        qsender.send(msg);
    }

    /**
     * Closes JMS objects.
     *
     * @throws JMSException if JMS fails to close objects due to internal error
     */
    public void close() throws JMSException {
        qsender.close();
        qsession.close();
        qcon.close();
    }

    /**
     * main() method.
     *
     * @param args WebLogic Server URL
     * @throws Exception if operation fails
     */
    public static void main(String[] args) throws Exception {
        if (args.length != 5) {
            System.out.println("Usage: java " + JmsSender.class.getName() +
                    " WebLogicURL user password QueueJNDIName QueueConnectionFactoryJNDIName");
            return;
        }
        JmsSender qs = new JmsSender();
        qs.init(args);
        readAndSend(qs);
        qs.close();
    }

    private static void readAndSend(JmsSender qs)
            throws IOException, JMSException {
        BufferedReader msgStream = new BufferedReader(new InputStreamReader(System.in));
        String line = null;
        boolean quitNow = false;
        do {
            System.out.print("\n\nType the message in the format:\n" +
                    "\t<Message text here>@<property-1=value-1,property-n=value-n>" +
                    "\n\nFor example:\nHello there!@DestinationRoute=creditService,AuthorizedPrincipal=app-1" +
                    "\n\nType (\"quit\" to quit): \n\n");
            line = msgStream.readLine();
            if (line != null && line.trim().length() != 0) {
                quitNow = line.equalsIgnoreCase("quit");
                if (!quitNow) {
                    qs.send(line);
                    System.out.println("JMS Message Sent: " + line + "\n");
                }
            }
        } while (!quitNow);

    }

}

