package org.bss.esb.transport;

import java.io.Serializable;

/**
 * A transport handler responsible for sending a message to a destination
 * for which is it configured. Depending on the implementation of this 
 * interface, the destination could be anything such as:
 * <pre>
 * 1) An Email address
 * 2) A FAX number
 * 3) An FTP location
 * 4) A WebService call over HTTP(S)
 * 5) Any other endpoint using a valid protocol
 * </pre>
 * 
 * Initialization of the <code>TransportHandler</code> is done via its
 * its setter methods as specified in the EsbConfiguration XML. The
 * implementation class of this interface must have one public
 * no-args constructor.
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 * 
 */
public interface TransportHandler extends Serializable {
    
    /**
     * Transports the message. The implementation class should be able to
     * handle the passed in message. This method should be invoked only after
     * this <code>TransportHandler</code> instance has been initialized.
     * 
     * @param message The message to be transported.
     * @exception TransportException Thrown if the handler fails to send
     * the passed in message to the destination specified while initializing
     * this handler instance. 
     */
    void transportMessage(Object message) throws TransportException;
    
}
