package org.bss.esb.transport.jms;

import org.bss.esb.transport.TransportException;
import org.bss.esb.transport.TransportHandler;

import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.logging.Logger;

/**
 * {@link TransportHandler} implementation for sending redelivery request
 * messages to a JMS Topic.
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 */
public class EsbRedeliveryHandler implements TransportHandler { 
    
    /** 
     * Used for obtaining the TopicConnection to beused for publishing 
     * redelivery request messages.
     */
    private TopicConnectionFactory tcf;
    
    /** Topic where the redelivery request messages will be published */
    private Topic topic;

    /** Logger instance used by this class */
    private Logger LOG = Logger.getLogger(getClass().getName());
    
    /** JNDI name of JMS topic connection factory */
    private String connectionFactory;
    
    /** JNDI name of JMS topic */
    private String destination;

    public String getConnectionFactory() {
        return connectionFactory;
    }

    public void setConnectionFactory(String connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    public String getDestination() {
        return destination;
    }

    public void setDestination(String destination) {
        this.destination = destination;
    }

    /**
     * Sends the JMS message to the configued JMS topic.
     * 
     * @param message The message to be sent.
     * @exception TransportException Thrown if the send fails.
     * @see TransportHandler#transportMessage(Object)
     */
    public void transportMessage(Object message) throws TransportException {
        
        TopicConnection con = null;
        
        try {
            if (tcf == null || topic == null) {
                
                InitialContext ic = new InitialContext();
                tcf = (TopicConnectionFactory) ic.lookup(connectionFactory);
                topic = (Topic) ic.lookup(destination);
                LOG.info("Looked up successfully:: TCF: "+connectionFactory+
                        "\tTopic: "+destination);

            }
            con = tcf.createTopicConnection();
            TopicSession sess = con.createTopicSession(false, 
                    TopicSession.AUTO_ACKNOWLEDGE);
            TopicPublisher pub = sess.createPublisher(topic);
            
            Long obj = (Long)message;
            Message msg = sess.createObjectMessage(obj);
            pub.publish(msg);
            
        } catch (JMSException je) {
            throw new TransportException(je);
        } catch (NamingException ne) {
            throw new TransportException(ne);
        } finally {
            try {

                con.close();
                
            } catch (Exception ignore) {
                
            }
        }
    }
} 
