package org.bss.esb.transport.jms;

import org.bss.esb.transport.TransportException;
import org.bss.esb.transport.TransportHandler;

import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.logging.Logger;

/**
 * {@link TransportHandler} implementation for sending messages
 * to a JMS Queue.
 * @version 1.0
 * @author Balwinder Sodhi
 */
public class JmsHandler implements TransportHandler { 
    
    /** JNDI name of the JMS queue connection factory to be used */
    private QueueConnectionFactory qcf;
    
    /** JMS queue where are messages are to be sent */
    private Queue queue;
    
    /** Logger instance to be used by this class */
    private Logger LOG = Logger.getAnonymousLogger();

    /** JNDI name of JMS queue connection factory */
    public String connectionFactory;
    
    /** JNDI name of JMS queue */
    public String destination;

    /**
     *
     * @return
     */
    public String getConnectionFactory() {
        return connectionFactory;
    }

    /**
     *
     * @param connectionFactory
     */
    public void setConnectionFactory(String connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     *
     * @return
     */
    public String getDestination() {
        return destination;
    }

    /**
     *
     * @param destination
     */
    public void setDestination(String destination) {
        this.destination = destination;
    }

    /**
     * Sends the JMS message to the configued JMS queue.
     * 
     * @param message The message to be sent.
     * @exception TransportException Thrown if the send fails.
     * @see TransportHandler#transportMessage(Object)
     */
    public void transportMessage(Object message) throws TransportException {
        
        QueueConnection con = null;
        
        try {
            if (qcf == null || queue == null) {
                InitialContext ic = new InitialContext();
                qcf = (QueueConnectionFactory) ic.lookup(connectionFactory);
                queue = (Queue) ic.lookup(destination);
                LOG.fine("Looked up successfully:: QCF: "+
                        connectionFactory+"\tQueue: "+destination);

            }
            con = qcf.createQueueConnection();
            QueueSession sess = con.createQueueSession(false, 
                    QueueSession.AUTO_ACKNOWLEDGE);
            QueueSender sender = sess.createSender(queue);
            sender.send((Message)message);
            
        } catch (JMSException je) {
            throw new TransportException(je);
        } catch (NamingException ne) {
            throw new TransportException(ne);
        } finally {
            try {

                con.close();
                
            } catch (Exception ignore) {
                
            }
        }
    }
} 
